using System;
using System.Linq;
using System.Threading.Tasks;
using Avalonia;
using Avalonia.Controls;
using Avalonia.Controls.ApplicationLifetimes;
using Avalonia.Markup.Xaml;
using Avalonia.Media;
using Avalonia.Platform.Storage;
using Tesses.CMS.Avalonia.ViewModels;
using Tesses.CMS.Avalonia.Views;
using Tesses.CMS.Client;
using Avalonia.Media.Imaging;
using Avalonia.Platform;
using Tesses.VirtualFilesystem;
using Tesses.VirtualFilesystem.Extensions;
using System.IO;
namespace Tesses.CMS.Avalonia;

public partial class App : Application
{
    public static IPlatform Platform {get;set;} = new NullPlatform();

    public static MainWindow? Window {get;set;}

    public static TessesCMSClient Client {get;} = new TessesCMSClient();
    
    public override void Initialize()
    {
        
        AvaloniaXamlLoader.Load(this);
    }

    public override void OnFrameworkInitializationCompleted()
    {
        Client.RootUrl = Platform.Configuration.ServerUrl;
        string title = GetApplicationTitle();
        if (ApplicationLifetime is IClassicDesktopStyleApplicationLifetime desktop)
        {
            Window = new MainWindow
            {
                DataContext = new MainViewModel(title),
                Title = title
            };
            desktop.MainWindow = Window;
        }
        else if (ApplicationLifetime is ISingleViewApplicationLifetime singleViewPlatform)
        {
            
            singleViewPlatform.MainView = new MainView
            {
                DataContext = new MainViewModel(title)
            };
        }

        base.OnFrameworkInitializationCompleted();
    }

    internal static string GetApplicationTitle()
    {
        string title="Tesses CMS";
        Task.Run(async()=>{
            try {
                var data = await Client.GetBrandingAsync();
                title = data.Title;
            }catch(Exception ex)
            {
                _=ex;
            }
        }).Wait();
        return title;
    }

    internal static async Task<IImage> GetMovieThumbnailAsync(string username, string name)
    {
        //we need to cache the resource
        if(Platform.Configuration.CacheResources && Platform.DownloadFilesystem != null)
        {
            UnixPath cacheDir = Special.Root / "Metadata" / "Cache" / username / "Movies" / name;
            Log($"About to create directory: {cacheDir}");
         
            await Platform.DownloadFilesystem.CreateDirectoryAsync(cacheDir);
            Log($"Created directory: {cacheDir}");
            UnixPath thumbnailPath = cacheDir / "thumbnail.jpg";
            
            if(await Platform.DownloadFilesystem.FileExistsAsync(thumbnailPath))
            {
                Log($"Image exists: {thumbnailPath}");
                using(var sr = await Platform.DownloadFilesystem.OpenReadAsync(thumbnailPath))
                {
                    MemoryStream ms = new MemoryStream();
                    sr.CopyTo(ms);
                    ms.Position=0;
                    Log($"Image read from file: {thumbnailPath}");
                    return new Bitmap(ms);
                }
                
            }
            else
            {
                var metadata = await Client.Movies.GetMovieContentMetadataAsync(username,name);
                if(metadata.HasThumbnail)
                {
                    using(var strm =  await Platform.DownloadFilesystem.OpenAsync(thumbnailPath,System.IO.FileMode.Create,System.IO.FileAccess.Write,System.IO.FileShare.None))
                    {
                        MemoryStream ms=new MemoryStream();
                        Log($"About to read from network and save: {thumbnailPath}");

                        await Client.Movies.DownloadThumbnailAsync(username,name,ms);
                        ms.Position=0;                        
                        ms.CopyTo(strm);
                        ms.Position=0;
                        Log($"Image read from network and saved: {thumbnailPath}");
                        return new Bitmap(ms);
                    }
                }
                
            }
        }
        else
        {
             var metadata = await Client.Movies.GetMovieContentMetadataAsync(username,name);
                if(metadata.HasThumbnail)
                {
                    MemoryStream ms=new MemoryStream();  
                        Log($"About to read from network: {username} {name}");
   
                    await Client.Movies.DownloadThumbnailAsync(username,name,ms);
                    ms.Position = 0;
                                            Log($"Image read from network: {username} {name}");

                    return new Bitmap(ms);
                    
                }
                
        }
        Log($"Image does not exist: {username} {name}");

        return new Bitmap(AssetLoader.Open(new Uri("avares://Tesses.CMS.Avalonia/Assets/no-media-icon.png")));
    }

    public static void Log(string text)
    {
        lock(App.Platform)
        {
            var fs=App.Platform.DownloadFilesystem;
            if(fs != null && App.Platform.Configuration.Log)
            {
                
                using(var strm = fs.Open(Special.Root/"Logs.txt",FileMode.Append,FileAccess.Write,FileShare.None))
                {
                    using(var sw = new StreamWriter(strm))
                    {
                        sw.WriteLine(text);
                    }
                }
            }
        }
    }
}